
########################################
# Run QOMET emulation experiment
#
# NOTE: This script uses the wireconf 
#       library to configure dummynet 
########################################

# verify the number of arguments
if [ $# -lt 8 ]
then
    echo "$(basename $0). Run QOMET emulation experiment."
    echo
    echo "ERROR: Not enough arguments were provided!"
    echo
    echo "Usage: $(basename $0) <test_name> <qomet_binary_output> <node_settings> <node1_id> <node1_ip> <node2_id> <node2_ip> <offered_load>"
    echo "(See the README file for details.)"
    exit 1
fi

#VALGRIND="valgrind --leak-check=full --track-origins=yes -v"

NO_WIRECONF=0
NO_PING=0
NO_IPERF=1

########################################
# Define test parameters
########################################

# test name
TEST_NAME=$1
OUTPUT_NAME=${TEST_NAME}

# QOMET output
QOMET_OUTPUT=$2

# node settings file and parameters
NODE_SETTINGS=$3
NODE1_ID=$4
NODE1_IP=$5
NODE2_ID=$6
NODE2_IP=$7

# offered load
OFFERED_LOAD=$8

########################################
# fixed parameters

PACKET_SIZE=1024
TEST_DURATION=5
INTERVAL=1

# tool for traffic generation
#IPERF=/usr/bin/iperf
IPERF=iperf
WIRECONF="/mnt/razvan_exfs1/work/qoala/qomet2/wireconf/wireconf -m"

# set id for server in unidirectional tests
SERVER_ID=0

# if the flag saying whether the test is bidirectional or not is undefined,
# set it to the default value "1"
if [ -z ${BIDIR_TEST} ]
then
BIDIR_TEST=1
fi


########################################
# Prepare the experiment
########################################
echo
echo "* Preparing experiment..."

########################################
# Start tcpdump

#echo "** Starting tcpdump..."
#sudo tcpdump -n -i lo0 -w ${output_name}_tcpdump.log &

########################################
# Configure alias interfaces and routes

# for FreeBSD
#sudo ifconfig lo0 inet 127.0.0.2 netmask 255.0.0.0 alias
#sudo route add -host 127.0.0.2 127.0.0.1 0
#sudo ifconfig lo0 inet 127.0.0.2 -alias

# for Linux
echo "** Configuring aliases..."
sudo ifconfig lo:0 127.0.0.2 up

########################################
# Prepare the system by loading dummynet
#echo
#echo "** Loading dummynet..."
# the command below is not needed if dummynet is compiled into the kernel
#sudo ./dummynet_load.sh


########################################
# Run the experiment
########################################

echo
echo "* Running experiment..."

if [ ${NO_IPERF} -ne 1 ]
then
if [ ${NODE1_ID} -eq ${SERVER_ID} ] || [ ${BIDIR_TEST} -eq 1 ]
then
########################################
# Start iperf server application
echo "** Starting iperf server..."
${IPERF} -s -u -i ${INTERVAL} -f m -l ${PACKET_SIZE} > ${OUTPUT_NAME}-iperf_s.log &
fi
fi

########################################
# Wait for server to start
echo "   Waiting 1 s for iperf server to start..."
sleep 1

echo
echo "****************************"
date
echo "****************************"

if [ ${NO_WIRECONF} -ne 1 ]
then
########################################
# Start the QOMET emulation
echo
echo "** Starting QOMET emulation..."
sudo ${VALGRIND} ${WIRECONF} -q ${QOMET_OUTPUT} -i ${NODE1_ID} -s ${NODE_SETTINGS} &
fi

########################################
# Wait for QOMET to start
#echo "   Waiting 1 s for QOMET to start..."
#sleep 1

if [ ${NODE1_ID} -ne ${SERVER_ID} ] || [ ${BIDIR_TEST} -eq 1 ]
then

if [ ${NO_IPERF} -ne 1 ]
then
########################################
# Start iperf client application
echo "** Starting iperf client..."
${IPERF} -c ${NODE2_IP} -u -i ${INTERVAL} -f k -l ${PACKET_SIZE} -b ${OFFERED_LOAD} -t ${TEST_DURATION} > ${OUTPUT_NAME}-iperf_c.log &
fi

if [ ${NO_PING} -ne 1 ]
then
########################################
# Start ping
echo "** Starting ping..."
PING_COUNT=$(echo "scale=0; ${TEST_DURATION}/${INTERVAL}+1"|bc)
sudo ping -i ${INTERVAL} -c ${PING_COUNT} -s ${PACKET_SIZE} ${NODE2_IP} > ${OUTPUT_NAME}-ping.log
else
# if node sends no ping, then we must sleep
sleep ${TEST_DURATION}
sleep 3
fi

else
# if node is passive (no iperf client and no ping), then we must sleep
sleep ${TEST_DURATION}
sleep 3
fi

echo
echo "****************************"
date
echo "****************************"


########################################
# Final actions
########################################

echo
echo "* Ending experiment..."
echo "  Waiting 5 s for finalization..."
sleep 5

########################################
# Kill processes

#echo "** Terminating ping..."
#sudo killall -INT ping

echo "** Killing iperf server..."
killall -INT iperf

#echo "** Killing tcpdump..."
#sudo killall tcpdump

########################################
# Clean the system

#echo "** Unloading dummynet..."
#sudo ./dummynet_unload.sh

# for Linux
echo "** Removing aliases..."
sudo ifconfig lo:0 down

########################################
# Process output files

#echo "** Processing output files..."
