
# Run experiment on 2 hosts to verify bandwidth emulation in QOMET
echo "TEST DESCRIPTION: Run experiment on 1 host using 2 interfaces to verify multi-interface emulation in QOMET"

# flag saying if the test is bidirectional (=1) or not (=0)
# NOTE: bidirectional tests are useful to demonstrate the effect of
#       contention handling in QOMET on bandwidth

BIDIR_TEST=0 # bidirectional tests not supported yet for this experiment

# set test parameters
TEST_NAME=$(basename $0)
QOMET_OUTPUT=test_wireconf-2if.xml.bin
NODE_SETTINGS=test_wireconf-2if.xml.settings
NODE1_ID=0
NODE1_IP=127.0.0.1
NODE2_ID=1
NODE2_IP=127.0.0.2
SCRIPT_PATH=/mnt/razvan_exfs1/work/qoala/qomet2/test_suite/wireconf

NODE1_MGMT_IP=172.16.72.23
NODE2_MGMT_IP=172.16.72.23

# set load higher than maximum available on 802.11b (around 6 Mbps)
# to verify bandwidth emulation in QOMET
# NOTE: queue filling up in dummynet will cause delay and loss results 
#       to be higher than expected
OFFERED_LOAD=10M

if [ ${BIDIR_TEST} -eq 0 ]
then
echo "TEST TYPE: Unidirectional"
echo "EXPECTED RESULTS (sample for 1->0):"
grep -v "datagrams received out-of-order" ${TEST_NAME}-0-iperf_s.log_unidir.valid | tail -1
else
echo "TEST TYPE: Bidirectional"
echo "EXPECTED RESULTS (sample for 1->0):"
grep -v "datagrams received out-of-order" ${TEST_NAME}-0-iperf_s.log_bidir.valid | tail -1
echo "EXPECTED RESULTS (sample for 0->1):"
grep -v "datagrams received out-of-order" ${TEST_NAME}-1-iperf_s.log_bidir.valid | tail -1
#grep -v "datagrams received out-of-order" ${TEST_NAME}-1-iperf_s.log_bidir.valid | tail -2
fi


# run experiment on node 1
# (-t forces TTY allocation; -f puts command in background)
ssh -tt -n -q ${NODE1_MGMT_IP} "cd ${SCRIPT_PATH} && env BIDIR_TEST=${BIDIR_TEST} ${SCRIPT_PATH}/run_wireconf_test.sh ${TEST_NAME}-0 ${SCRIPT_PATH}/${QOMET_OUTPUT} ${SCRIPT_PATH}/${NODE_SETTINGS} ${NODE1_ID} ${NODE1_IP} ${NODE2_ID} ${NODE2_IP} ${OFFERED_LOAD} > ${TEST_NAME}-0.log 2>&1" &

echo

# run experiment on node 2
ssh -t -q ${NODE2_MGMT_IP} "cd ${SCRIPT_PATH} && env BIDIR_TEST=${BIDIR_TEST} ${SCRIPT_PATH}/run_wireconf_test.sh ${TEST_NAME}-1 ${SCRIPT_PATH}/${QOMET_OUTPUT} ${SCRIPT_PATH}/${NODE_SETTINGS} ${NODE2_ID} ${NODE2_IP} ${NODE1_ID} ${NODE1_IP} ${OFFERED_LOAD} > ${TEST_NAME}-1.log 2>&1"

echo "[NOTE: Please ignore the ssh error 'tcgetattr: Inappropriate ioctl for device' which seems to be harmless.]"
echo

# wait for log files to be ready
sleep 2

echo "ACTUAL RESULTS (1->0):"
grep -v "datagrams received out-of-order" ${TEST_NAME}-0-iperf_s.log | tail -1
if [ ${BIDIR_TEST} -eq 1 ]
then
echo "ACTUAL RESULTS (0->1):"
grep -v "datagrams received out-of-order" ${TEST_NAME}-1-iperf_s.log | tail -1
fi

