
/*
 * Copyright (c) 2006-2013 The StarBED Project  All rights reserved.
 *
 * See the file 'LICENSE' for licensing information.
 *
 */

/************************************************************************
 *
 * QOMET Emulator Implementation
 *
 * File name: ipfw_wrap.h
 * Function: Header file of ipfw wrapper of the wireconf library
 *
 * Authors: Junya Nakata, Razvan Beuran
 *
 ***********************************************************************/

#ifndef	__IPFW_WRAP_H
#define	__IPFW_WRAP_H


#include <sys/queue.h>
#include <sys/socket.h>

#include <sys/types.h>

#include <net/if.h>
//#include <netinet/in.h>

#include <pthread.h>


#include "routing_info.h"

//#include <netinet/ip_fw.h>
//#include <netinet/ip_dummynet.h>
#include "../ipfw3/ipfw/include_e/netinet/ip_fw.h"
#include  "../ipfw3/ipfw/include_e/netinet/ip_dummynet.h"
#include "../ipfw3/glue.h"


/////////////////////////////////////////////
// Constants
/////////////////////////////////////////////

#define MAX_BANDWIDTH  1e9
#define MAX_DELAY      1e3
#define MAX_LOSS_RATE  1.0


/////////////////////////////////////////////
// Socket manipulation functions
/////////////////////////////////////////////

// get socket identifier;
// return socket identifier on success, ERROR on error
int get_socket (void);

// close socket specified by socket_id
void close_socket (int socket_id);


////////////////////////////////////////////////
// ipfw rule related functions
////////////////////////////////////////////////

// add an ipfw rule containing a dummynet pipe to the firewall;
// rule number is 'rulenum', pipe number is 'pipe_nr', source and 
// destination are 'src' and 'dst', direction is 'direction';
// return SUCCESS on success, ERROR on error

// NOTE: both 'src' and 'dst' can be either specific 
// IP addresses or the string "any"
int add_rule_and_pipe (int socket_id, uint16_t rulenum, int pipe_nr,
		       char *src, char *dst, int direction);
int add_rule_and_pipe_mac (int socket_id, uint16_t rulenum, int pipe_nr,
			   char *src, char *dst, int direction);

// get ipfw rules; on return, rule data will be stored in 'rule_data';
// if 'rule_data' must be allocated, set 'rule_data_alloc_size' to
// UNDEFINED_RULE_DATA_SIZE when calling, and on return it wil contain
// the allocated size, otherwise it is assumed 'rule_data_alloc_size'
// contains the allocate size of 'rule_data' (this is useful for next
// call, to avoid reallocation each time); on return 'rules' is an array of
// pointers to consecutive rules, and 'rules_no' is the number of rules;
// return SUCCESS on success, ERROR on error
int get_rules (uint socket_id, struct ip_fw **rule_data,
	       int *rule_data_alloc_size, struct ip_fw **rules, int *rule_no);

// delete an ipfw rule specified by 'rule_number';
// return SUCCESS on success, ERROR on error
int delete_rule (uint s, u_int32_t rule_number);

// print a rule structure given in 'rule'
void print_rule (struct ip_fw *rule);


////////////////////////////////////////////////
// dummynet pipe related functions
////////////////////////////////////////////////

// configure a dummynet pipe identified by 'pipe_nr';
// the parameters to use are 'bandwidth' [bps], 'delay' [ms],
// 'loss_rate' [probability], and 'queue_size' [packets];
// return SUCCESS on success, ERROR on error
int
configure_pipe (int socket_id, int pipe_nr, float bandwidth,
		float delay, float loss_rate, int queue_size);

// delete an ipfw pipe identified by 'pipe_number' by using a system call;
// return SUCCESS on success, ERROR on error
int delete_pipe_syscall (int pipe_number);

// delete an ipfw pipe identified by 'pipe_number' by using socket operations;
// return SUCCESS on success, ERROR on error
int delete_pipe (uint s, int pipe_number);

// print a pipe structure given by 'link' and 'scheduler'
void print_pipe (struct dn_link *link, struct dn_fs *scheduler);

#endif
