
/*
 * Copyright (c) 2006-2013 The StarBED Project  All rights reserved.
 *
 * See the file 'LICENSE' for licensing information.
 *
 */

/************************************************************************
 *
 * QOMET Emulator Implementation
 *
 * File name: routing_info.h
 * Function: Header file of routing_info.c 
 *
 * Authors: Lan Nguyen Tien, Razvan Beuran
 *
 *   $Revision: 128 $
 *   $LastChangedDate: 2009-02-06 10:21:50 +0900 (Fri, 06 Feb 2009) $
 *   $LastChangedBy: razvan $
 *
 ***********************************************************************/

#ifndef	__ROUTING_INFO_H
#define	__ROUTING_INFO_H

#include <sys/types.h>
#include <sys/socket.h>
#include <net/route.h>
//#include <sys/stddef.h>
#include <linux/stddef.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
//#include <net/if_dl.h>
#include <sys/param.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <linux/rtnetlink.h>

/////////////////////////////////////////////
// Basic constants
/////////////////////////////////////////////

#ifdef __FreeBSD__
// size of buffer used when getting route information from kernel
#define ROUTE_BUFFER_SIZE (sizeof(struct rt_msghdr) + 512)
#elif __linux__
// size of buffer used when getting route information from kernel
#define ROUTE_BUFFER_SIZE (sizeof(struct rtmsg) + 512)

#define BUFFER_SIZE 8192

// buffer to hold the RTNETLINK request
struct rtnetlink_request
{
  struct nlmsghdr nl;
  struct rtmsg rt;
  char buf[BUFFER_SIZE];
};

#endif

// constant used in route message structure
#define RTM_SEQ_CT                    9999


/////////////////////////////////////////////
// Basic macros
/////////////////////////////////////////////

// roundup socket address length (see below)
#define ROUNDUP(a, size) (((a) & ((size)-1)) ? (1 + ((a) | ((size)-1))) : (a))

// get next socket address
#define NEXT_SA(ap) ap = (struct sockaddr *) \
((caddr_t) ap+ (ap->sa_len ? ROUNDUP(ap->sa_len, sizeof(u_long)) : \
sizeof(u_long)))


/////////////////////////////////////////////
// Routing table related functions
/////////////////////////////////////////////

// given the arrays of IP addresses in binary and char format
// 'IP_addresses' and 'IP_char_addresses', get the next hop id
// for the route to destination 'dst_id'; 'direction' can be either
// ROUTE_INCOMING or ROUTE_OUTGOING;
// return the next_hop_id on success, ERROR on error
int get_next_hop_id (in_addr_t * IP_addresses, char *IP_char_addresses,
		     int id_dst, int direction);

// FreeBSD version
// get from the routing table the next hop associated to destination with
// IP address 'dst'; the result IP address is stored in the argument 
// 'next_hop';
// return SUCCESS on success, ERROR on error
int get_next_hop_ip_freebsd (char *dst, char *next_hop);

// Linux version
// get from the routing table the next hop associated to destination with
// IP address 'dst'; the result IP address is stored in the argument 
// 'next_hop';
// return SUCCESS on success, ERROR on error
int get_next_hop_ip_linux (char *dst, char *next_hop);


#ifdef __FreeBSD__
// get addresses from routing table
void get_routing_table_addr (int addrs, struct sockaddr *sa,
			     struct sockaddr **rti_info);
#endif

// transform a socket adress 'sa' with length 'salen'
// into a readable IP address (string);
// return the IP address as a string
char *sock_ntop_host (const struct sockaddr *sa, socklen_t salen);

// map an IP address string to internal QOMET node id;
// return id on success, -1 on error
int get_id_from_ip_string (in_addr_t * IP_addresses, char *s);

// map an IP address value given as in_addr_t to internal QOMET node id;
// return id on success, ERROR on error
int get_id_from_ip_in_addr_t (struct in_addr *IP_addresses, in_addr_t t);

// map an internal QOMET node id to an IP address; 
// the address is return in the argument 'ip';
// return SUCCESS on success, ERROR on error
int get_ip_from_id (char *IP_char_addresses, int id, char *ip);

// read settings (node ids and corresponding IP adresses)
// from a file, and store the adresses in the array p at
// the corresponding index;
// return the number of addresses (=nodes) successfully read, 
// or -1 on ERROR
//int read_ip_settings (char *path, in_addr_t * p, char * p_char, int p_size);

#endif
