
/*
 * Copyright (c) 2006-2013 The StarBED Project  All rights reserved.
 *
 * See the file 'LICENSE' for licensing information.
 *
 */

/************************************************************************
 *
 * QOMET Emulator Implementation
 *
 * File name: wireconf.h
 * Function: Header file of wired-network emulator configurator library
 *
 * Authors: Junya Nakata, Razvan Beuran
 *
 ***********************************************************************/

#ifndef	__WIRECONF_H
#define	__WIRECONF_H


#include <sys/queue.h>
#include <sys/socket.h>

#include <sys/types.h>

#include <net/if.h>
//#include <netinet/in.h>

#include <pthread.h>


#include "routing_info.h"
#include "ipfw_wrap.h"
#include "wireconf_global.h"
#include "wireconf_message.h"

//#include <netinet/ip_fw.h>
//#include <netinet/ip_dummynet.h>
#include "../ipfw3/ipfw/include_e/netinet/ip_fw.h"
#include  "../ipfw3/ipfw/include_e/netinet/ip_dummynet.h"
#include "../ipfw3/glue.h"

#include "../deltaQ/deltaQ.h"


/////////////////////////////////////////////
// Basic constants
/////////////////////////////////////////////

// traffic directions to which dummynet configuration
// is to be applied
#define DIRECTION_BOTH                  0
#define DIRECTION_IN                    1
#define DIRECTION_OUT                   2

// default size of pipe queue
//#define PIPE_QSIZE                      50
#define PIPE_QSIZE                      1000

#define UNDEFINED_ALLOC_SIZE            -1

#define MAX_NODES_W                     100

// after recent changes to do_wireconf for binary 
// support, it is not safe to assume first node id
// can be different than 0 !!!!!!!!!!!!!!!!!!!!
#define FIRST_NODE_ID                   0


///////////////////////////////////////////////
// IPv4 address data structure
///////////////////////////////////////////////

// IPv4 address structure
union in4_addr
{
  uint8_t octet[4];
  uint32_t word;
} in4_addr;


//////////////////////////////////
// Wireconf structure
//////////////////////////////////

// wireconf class structure
struct wireconf_class
{
  // the starting time for this wireconf module
  struct timespec start_time;

  // my id in the experiment
  int my_id;

  // experiment-side IP addresses for all nodes in the experiment
  struct in_addr *IP_addresses;

  // the number of nodes in the experiment
  int node_count;

  // thread for statistics communication
  pthread_t stats_send_thread;

  // flag showing that send thread execution must be stopped
  int do_interrupt_send;

  // pipe for providing statistics to send thread
  int stats_provide_pipe[2];

  // thread for listening to statistics information
  pthread_t stats_listen_thread;

  // flag showing that listen thread execution must be stopped
  int do_interrupt_listen;

  // array of total channel utilizations for each source
  float *total_channel_utilizations;

  // array of total transmission probabilities for each source
  float *total_transmission_probabilities;

  // array of channel utilizations from this node to each destination
  float *self_channel_utilizations;

  // array of transmission probabilities from this node to each destination
  float *self_transmission_probabilities;

  float total_channel_utilization_others;
  float total_transmission_probability_others;
  float total_self_number_packets;

  //Lan: array of channel utilization per source of all other nodes
  float **channel_utilization_per_source;

  /* NOTE: the code below intended for the next version of wireconf,
     that will be able to read QOMET binary output

     // data queue of this chanel
     data_queue_class data_queue;

     // array of flags showing which destinations are
     // active (TRUE=>active, FALSE=>inactive)
     int active_destinations[MAX_DESTINATIONS];

     // configuration array
     deltaQ_class configurations[MAX_DESTINATIONS];
   */

};

#endif
